"A performer from a scraping operation..."
type ScrapedPerformer {
  "Set if performer matched"
  stored_id: ID
  name: String
  disambiguation: String
  gender: String
  url: String @deprecated(reason: "use urls")
  urls: [String!]
  twitter: String @deprecated(reason: "use urls")
  instagram: String @deprecated(reason: "use urls")
  birthdate: String
  ethnicity: String
  country: String
  eye_color: String
  height: String
  measurements: String
  fake_tits: String
  penis_length: String
  circumcised: String
  career_length: String
  tattoos: String
  piercings: String
  # aliases must be comma-delimited to be parsed correctly
  aliases: String
  tags: [ScrapedTag!]

  "This should be a base64 encoded data URL"
  image: String @deprecated(reason: "use images instead")
  images: [String!]
  details: String
  death_date: String
  hair_color: String
  weight: String
  remote_site_id: String
}

input ScrapedPerformerInput {
  "Set if performer matched"
  stored_id: ID
  name: String
  disambiguation: String
  gender: String
  url: String @deprecated(reason: "use urls")
  urls: [String!]
  twitter: String @deprecated(reason: "use urls")
  instagram: String @deprecated(reason: "use urls")
  birthdate: String
  ethnicity: String
  country: String
  eye_color: String
  height: String
  measurements: String
  fake_tits: String
  penis_length: String
  circumcised: String
  career_length: String
  tattoos: String
  piercings: String
  aliases: String

  # not including tags for the input
  # not including image for the input
  details: String
  death_date: String
  hair_color: String
  weight: String
  remote_site_id: String
}
