input MigrateSceneScreenshotsInput {
  # if true, delete screenshot files after migrating
  deleteFiles: Boolean
  # if true, overwrite existing covers with the covers from the screenshots directory
  overwriteExisting: Boolean
}

input MigrateBlobsInput {
  # if true, delete blob data from old storage system
  deleteOld: Boolean
}
