type SQLQueryResult {
  "The column names, in the order they appear in the result set."
  columns: [String!]!
  "The returned rows."
  rows: [[Any]!]!
}

type SQLExecResult {
  """
  The number of rows affected by the query, usually an UPDATE, INSERT, or DELETE.
  Not all queries or databases support this feature.
  """
  rows_affected: Int64
  """
  The integer generated by the database in response to a command.
  Typically this will be from an "auto increment" column when inserting a new row.
  Not all databases support this feature, and the syntax of such statements varies.
  """
  last_insert_id: Int64
}
