type StashBox {
  endpoint: String!
  api_key: String!
  name: String!
  max_requests_per_minute: Int!
}

input StashBoxInput {
  endpoint: String!
  api_key: String!
  name: String!
  # defaults to 240
  max_requests_per_minute: Int
}

type StashID {
  endpoint: String!
  stash_id: String!
  updated_at: Time!
}

input StashIDInput {
  endpoint: String!
  stash_id: String!
  updated_at: Time
}

input StashBoxFingerprintSubmissionInput {
  scene_ids: [String!]!
  stash_box_index: Int @deprecated(reason: "use stash_box_endpoint")
  stash_box_endpoint: String
}

input StashBoxDraftSubmissionInput {
  id: String!
  stash_box_index: Int @deprecated(reason: "use stash_box_endpoint")
  stash_box_endpoint: String
}
